"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.format = format;
var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));
var _react = require("react");
var _tokenizer = require("./tokenizer");
function format(plainText, tags, defaultTag, keepUnknownTags, tokens, index, hooks) {
  if (!Array.isArray(tokens)) {
    return formatToken(plainText, tags, defaultTag, keepUnknownTags, tokens, index, hooks);
  }
  var elems = tokens.map(function (token, index) {
    return format(plainText, tags, defaultTag, keepUnknownTags, token, index, hooks);
  });
  return /*#__PURE__*/(0, _react.createElement)(_react.Fragment, {
    key: index
  }, elems);
}
function formatToken(plainText, tags, defaultTag, keepUnknownTags, token, index, hooks) {
  var elem;
  if (_tokenizer.Tokenizer.isTextToken(token)) {
    elem = plainText(index, token, hooks);
  } else {
    var handler = tags[token.tag];
    var elems = token.children.map(function (child, index) {
      return format(plainText, tags, defaultTag, keepUnknownTags, child, index, hooks);
    });
    var tagData = {
      name: token.tag,
      children: elems,
      attrs: token.attrs
    };
    if (handler) {
      return handler(index, tagData, hooks);
    } else if (defaultTag) {
      return defaultTag(index, tagData, hooks);
    }
    return keepUnknownTags ? tagTokenToString(token.tag, token.attrs, elems, index) : /*#__PURE__*/(0, _react.createElement)(_react.Fragment, {
      key: index
    }, elems);
  }
  return /*#__PURE__*/(0, _react.createElement)(_react.Fragment, {
    key: index
  }, elem);
}
function tagTokenToString(tag, attrs, children, index) {
  var attrsText = attrsToString(attrs);
  var content = "<".concat(tag).concat(attrsText ? " ".concat(attrsText) : '').concat(children.length ? '>' : attrsText.length ? ' />' : '/>');
  var openTag = /*#__PURE__*/(0, _react.createElement)('pre', {
    key: 0,
    style: {
      display: 'inline'
    }
  }, content);
  var closeTag = children.length ? /*#__PURE__*/(0, _react.createElement)('pre', {
    key: 2,
    style: {
      display: 'inline'
    }
  }, "</".concat(tag, ">")) : '';
  return /*#__PURE__*/(0, _react.createElement)(_react.Fragment, {
    key: index
  }, [openTag, children, closeTag]);
}
function attrsToString(attrs) {
  return Object.entries(attrs).reduce(function (res, _ref) {
    var _ref2 = (0, _slicedToArray2["default"])(_ref, 2),
      key = _ref2[0],
      value = _ref2[1];
    res.push("".concat(key, "=\"").concat(value, "\""));
    return res;
  }, []).join(' ');
}