"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createTextFormat = createTextFormat;
var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));
var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));
var _react = require("react");
var _tokenizer = require("./helpers/tokenizer");
var _format = require("./helpers/format");
/**
 * Definition of a handler to render unformatted text.
 *
 * Note that it stills need to specify `key={index}` as it might be called
 * as part of another tag content together with other children.
 */

/**
 * Definition of a handler to render styled text
 *
 * Note that `key={index}` is needed because they might be rendered
 * together with other children as part of another tag content
 */

/**
 * This _Component Factory_ accepts a configuration and returns a component that
 * can be used to render formatted strings from plain text converting it to
 * native React components.
 * Think about it as a controlled/styled HTML-like tags.
 */
function createTextFormat(config) {
  var _config$textHandler;
  var keepUnknownTags = Boolean(config.keepUnknownTags);
  var textHandler = (_config$textHandler = config.textHandler) !== null && _config$textHandler !== void 0 ? _config$textHandler : function (index, text) {
    return /*#__PURE__*/(0, _react.createElement)(_react.Fragment, {
      key: index
    }, text);
  };
  var uppercasedTagHandlers = transformTagHandlers(config);
  var getHooks = config.hooks;
  var TextFormat = function TextFormat(props) {
    var hooks = getHooks === null || getHooks === void 0 ? void 0 : getHooks();
    // parsing text *might* be considered a heavy operation so we cache it
    // `props.children` is used to make sure it depends on the value, not the
    // props object which can change by mistake depending on how the component
    // is used (a.k.a., not using React properly ^^;)
    var formattedText = (0, _react.useMemo)(function () {
      var text = props.children || '';
      var tokens = new _tokenizer.Tokenizer().parse(text);
      return (0, _format.format)(textHandler, uppercasedTagHandlers, config.defaultTagHandler, keepUnknownTags, tokens, 0, hooks);
    }, [props.children].concat((0, _toConsumableArray2["default"])(hooks || [])));
    return formattedText;
  };
  return TextFormat;
}

/**
 * Dummy-proof function to ensure that all the tag names are
 * provided in uppercase
 */
function transformTagHandlers(config) {
  if (!config.tagHandlers) {
    return {};
  }
  return Object.entries(config.tagHandlers).reduce(function (acc, _ref) {
    var _ref2 = (0, _slicedToArray2["default"])(_ref, 2),
      tag = _ref2[0],
      handler = _ref2[1];
    acc[tag.toUpperCase()] = handler;
    return acc;
  }, {});
}